from flask import render_template, g, redirect, url_for, flash
from flask_wtf.form import FlaskForm
from sqlalchemy.orm import joinedload
import werkzeug.exceptions
import wtforms

import mo
import mo.db as db
from mo.jobs import TheJob, job_file_size
from mo.web import app
import mo.web.util


class JobDeleteForm(FlaskForm):
    delete_job_id = wtforms.IntegerField()
    delete = wtforms.SubmitField('Smazat')


@app.route('/org/jobs/', methods=('GET', 'POST'))
def org_jobs():
    sess = db.get_session()

    form_delete_job = JobDeleteForm()
    if form_delete_job.validate_on_submit():
        tj = TheJob(form_delete_job.delete_job_id.data)
        job = tj.load()
        if not job:
            flash('Dávka mezitím zmizela.', 'success')
        elif not (job.user_id == g.user.user_id or g.user.is_admin):
            flash('Tuto dávku nemáte právo smazat', 'danger')
        elif job.state == db.JobState.running:
            flash('Běžící dávku nelze smazat', 'danger')
        else:
            tj.remove_loaded()
            flash('Dávka smazána', 'success')
        return redirect(url_for('org_jobs'))

    job_query = (sess.query(db.Job)
                 .options(joinedload(db.Job.user)))
    if not g.user.is_admin:
        job_query = job_query.filter_by(user=g.user)

    jobs = job_query.order_by(db.Job.created_at.desc()).all()

    return render_template(
        'org_jobs.html',
        jobs=jobs,
        form_delete_job=form_delete_job,
    )


def get_job(id: int) -> db.Job:
    job = db.get_session().query(db.Job).options(joinedload(db.Job.user)).get(id)
    if job is None:
        raise werkzeug.exceptions.NotFound()

    if not (job.user_id == g.user.user_id or g.user.is_admin):
        raise werkzeug.exceptions.Forbidden()

    return job


@app.route('/org/jobs/<int:id>/')
def org_job(id: int):
    job = get_job(id)

    has_errors = (job.state == db.JobState.failed
                  and isinstance(job.out_json, dict)
                  and 'errors' in job.out_json)

    return render_template(
        'org_job.html',
        job=job,
        has_errors=has_errors,
        in_size=job_file_size(job.in_file),
        out_size=job_file_size(job.out_file),
    )


@app.route('/org/jobs/<int:id>/output')
def org_job_output(id: int):
    job = get_job(id)
    return mo.web.util.send_job_result(job)
